import os
import xml.etree.ElementTree as ET
from ansible.module_utils.basic import AnsibleModule


# replace default update center with our own
def set_default(tree, url):
    default_update_center = tree.find(".//site[id='default']")
    if default_update_center is None:
        raise Exception('Default update center not found')
    new_url = default_update_center.find('url')
    if new_url.text != url:
        new_url.text = url
        return True
    return False


# append additional update centers
def append(tree, site_id, url):
    xpath = './/site[id="{0}"]'.format(site_id)
    check_existing = tree.find(xpath)
    if check_existing is None:
        # Append if not found
        for sites in tree.iter('sites'):
            site = ET.Element('site')
            new_site_id = ET.SubElement(site, 'id')
            new_site_id.text = site_id
            new_url = ET.SubElement(site, 'url')
            new_url.text = url
            sites.append(site)
        return True
    else:
        # Update URL if necessary
        url_node = check_existing.find('url')
        if url_node is not None and url_node.text != url:
            url_node.text = url
            return True
        if url_node is None:
            ET.SubElement(check_existing, 'url').text = url
            return True
        return False


def main():
    module = AnsibleModule(
        argument_spec={
            'jenkins_home': {'default': '/var/lib/jenkins'},
            'update_center_id': {'required': True},
            'update_center_url': {'required': True}
        }
    )
    update_ctr_config = os.path.join(module.params['jenkins_home'],
                                     'hudson.model.UpdateCenter.xml')
    tree = ET.parse(update_ctr_config)
    if module.params['update_center_id'] == 'default':
        changed = set_default(tree, module.params['update_center_url'])
    else:
        changed = append(tree,
                         module.params['update_center_id'],
                         module.params['update_center_url'])
    if changed:
        tree.write(update_ctr_config, encoding='UTF-8')
    module.exit_json(changed=changed)


main()



from __future__ import print_function
from argparse import ArgumentParser, REMAINDER
from os import getcwd, path
from wrappers import call_ansible

import sys


def cinch_generic(playbook, help_description):
    # Parse the command line arguments
    parser = ArgumentParser(description='A CLI wrapper for ansible-playbook '
                            'to run cinch playbooks.  ' + help_description)
    # The inventory file that the user provides which will get passed along to
    # Ansible for its consumption
    parser.add_argument('inventory', help='Ansible inventory file (required)')
    # All remaining arguments are passed through, untouched, to Ansible
    parser.add_argument('args', nargs=REMAINDER, help='extra args to '
                        'pass to the ansible-playbook command (optional)')
    args = parser.parse_args()
    if len(args.inventory) > 0:
        if args.inventory[0] == '/':
            inventory = args.inventory
        else:
            inventory = path.join(getcwd(), args.inventory)
    else:
        raise Exception('Inventory path needs to be non-empty')
    exit_code = call_ansible(inventory, playbook, args.args)
    sys.exit(exit_code)


def cinch():
    """
    Entry point for the "cinch" CLI that merely wraps the ansible-playbook
    command and pre-fills its path to the site.yml file for Cinch. The cinch
    tool requires a single argument - the Ansible inventory file - and accepts
    an arbitrary number of extra arguments that are passed through to the
    ansible-playbook executable.

    :return: Exit code 0 if the execution is completed successfully, or 255
    if an unknown error occurs. If ansible-playbook exits with an error code,
    this executable will exit with the same code.
    """
    help_description = '''This command runs the 'site.yml' playbook to
                          configure a Jenkins master or slave.'''
    cinch_generic('site.yml', help_description)


def teardown():
    """
    Entry point for the "teardown" CLI that wraps ansible-playbook commands and
    pre-fills its path to the teardown.yml file.

    :return: Exit code 0 if the execution is completed successfully, or 255 if
    an unknown error occurs. If ansible-playbook exits with an error code, this
    executable will exit with the same code.
    """
    help_description = '''This command runs the 'teardown.yml' playbook to
                          disconnect a Jenkins slave.'''
    cinch_generic('teardown.yml', help_description)


if __name__ == '__main__':
    print('You should not invoke this file directly.')
    sys.exit(1)



#!/usr/bin/env python
#
# Author: Greg Hellings - <ghelling@redhat.com> or <greg.hellings@gmail.com>
#
# Module to configure users in Jenkins authorized to use CLI
import xml.etree.ElementTree as ET
import os
from ansible.module_utils.basic import AnsibleModule


DOCUMENTATION = '''
---
version_added: "2.1"
module: jenkins_cli_user
short_description: configure Jenkins CLI users with pub key
description:
  - This module configures admin users in Jenkins to utilize the specified
    SSH pubkey. Requires that role-based authentication be enabled and that
    a user be configured as an admin

options:
  jenkins_home:
    description:
     The root directory for the Jenkins install
    required: true
  jenkins_user:
    description:
      The name of the user to configure the SSH key for
  key_file:
    description:
      Path to the SSH keyfile to be listed as authorized
    required: true
  state:
    description:
      Currently limited to "present" - will create the user
    required: false

author: Gregory Hellings
'''


def main():
    module = AnsibleModule(
        argument_spec={
            'jenkins_home': {'required': True},
            'jenkins_user': {'required': True},
            'key_file': {'required': True},
            'state': {'choices': ['present'], 'default': 'present'}
        },
        supports_check_mode=False
    )
    params = type('Params', (object,), module.params)
    user_config_path = os.path.join(params.jenkins_home, "users")
    changed = False
    # This is the local SSH key file to read and enter into the Jenkins config
    # for the desginated user
    with open(params.key_file) as key:
        pub_key = key.read()
        user_cfg_file = os.path.join(user_config_path, params.jenkins_user,
                                     "config.xml")
        # Parsing the XML structure of the per-user configuration file, in
        # order to add the SSH keys
        usertree = ET.parse(user_cfg_file)
        userroot = usertree.getroot()
        keyroot = userroot.find("properties")
        keys = keyroot.getiterator("authorizedKeys")
        if keys:
            for key in keys:
                # The value of key.text is the appended public SSH keys that
                # should be usable by this character, separated each by a \n
                # character. Very much like an authoritzed_keys file for SSH
                # access. If this key is not a substring of the whole set of
                # ahtorized keys, then we append it to the list of keys and set
                # changed to true
                if pub_key not in str(key.text):
                    changed = True
                    if key.text is None:
                        key.text = pub_key
                    else:
                        key.text = str(key.text) + pub_key
        else:
            # No authorized keys have been added to the user hitherto, and even
            # the XML structure that holds the set of authorized keys is not
            # present, so we must create the XML structure to hold the
            # requested key. In this case, the module is definitely making a
            # change to the files
            changed = True
            prop = userroot.find("properties")
            ssh_auth = ET.SubElement(prop,
                                     "org.jenkinsci.main.modules"
                                     ".cli.auth.ssh."
                                     "UserPropertyImpl")
            auth_key = ET.SubElement(ssh_auth, "authorizedKeys")
            auth_key.text = pub_key
        if changed:
            usertree.write(user_cfg_file, encoding="UTF-8")
        module.exit_json(changed=changed)
    # This code should only be accessed if there was an exception within the
    # "with" block that prevents the module.exit_json line above from being
    # properly executed. Then this will return an error to the user, rather
    # than leaving them completely hanging.
    module.fail_json(msg="Roles not found - have you configured an admin "
                     "using the Role-based Authorization Strategy?")


main()



import os
import unittest

from cinch.bin.wrappers import call_ansible, command_handler


class CinchCLI(unittest.TestCase):

    def test_exit_code_zero(self):
        # full path to the playbook and inventory must be specified in order to
        # avoid path issues with the BASE variable in wrappers.py
        playbook = os.path.join(os.getcwd(), 'tests/playbook.yml')
        inventory = os.path.join(os.getcwd(), 'tests/inventory.ini')

        # valid inventory and playbook files
        self.assertEqual(call_ansible(inventory, playbook), 0)

    def test_exit_code_one(self):
        # inventory and playbook files that do not exist
        self.assertEqual(call_ansible('junk.ini', 'junk.yml'), 1)

    def test_exit_code_255(self):
        # invalid data given to plumbum which should return exit code 255
        self.assertEqual(command_handler('invalid_command',
                                         'invalid_arg'), 255)



from ansiblelint import AnsibleLintRule
try:
    from types import StringTypes
except ImportError:
    # Python3 removed types.StringTypes
    StringTypes = str,


class NoFormattingInWhenRule(AnsibleLintRule):
    id = 'ANSIBLE0019'
    shortdesc = 'No Jinja2 in when/until'
    description = '"when" or "until" lines should not include Jinja2 variables'
    tags = ['deprecated']

    def _is_valid(self, when):
        if not isinstance(when, StringTypes):
            return True
        return when.find('{{') == -1 and when.find('}}') == -1

    def matchplay(self, file, play):
        errors = []
        if isinstance(play, dict):
            if 'roles' not in play:
                return errors
            for role in play['roles']:
                if self.matchtask(file, role):
                    errors.append(({'when': role},
                                   'role "when" clause has Jinja2 templates'))
        if isinstance(play, list):
            for play_item in play:
                sub_errors = self.matchplay(file, play_item)
                if sub_errors:
                    errors = errors + sub_errors
        return errors

    def matchtask(self, file, task):
        return ('when' in task and not self._is_valid(task['when'])) or \
               ('until' in task and not self._is_valid(task['until']))



#!/usr/bin/env python

import re
from ansible.module_utils.basic import AnsibleModule
from os import path


def main():
    module = AnsibleModule(
        argument_spec={
            'file': {'required': True},
            'line': {'required': False},
            'method': {'choices': ['regex', 'simple'], 'default': 'simple'}
        }
    )
    # First sanity check that the file exists and is not a directory or such
    if not path.exists(module.params['file']) or \
       not path.isfile(module.params['file']):
        module.exit_json(changed=False, exists=False, present=False)
    # Create the method that will do the matching
    if module.params['method'] == 'regex':
        expression = re.compile(module.params['line'])

        def matcher(x):
            return expression.search(x) is not None
    else:
        def matcher(x):
            return x == module.params['line']
    # Read the file, line by line, and check for matches
    with open(module.params['file'], 'r') as reader:
        for line in reader.readlines():
            if matcher(line):
                module.exit_json(changed=False, exists=True, present=True)
    module.exit_json(changed=False, exists=True, present=False)


main()



from __future__ import print_function

from plumbum import local
from plumbum.commands.processes import ProcessExecutionError
from traceback import print_exc

import os
import sys


BASE = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
# Link to our docs with configuration examples
DOCS = 'https://redhatqe-cinch.readthedocs.io/en/latest/users.html'
# Skeleton text to insert in YAML config files
SKEL_TEXT = '''---
# Add your cinch {0} configuration here
# Examples: {1}
'''


def call_ansible(inventory, playbook, *args):
    """
    Wraps a call out to the ansible-playbook executable, passing it the cinch
    site.yml file that kicks off this playbook.

    :param inventory: The Ansible inventory file to pass through
    :param args: An array of other command-line arguments to ansible-playbook
    to pass
    :return: The exit code returned from ansible-playbook, or 255 if errors
    come from elsewhere
    """
    # Construct the arguments to pass to Ansible by munging the arguments
    # provided to this method
    ansible_args = [
        os.path.join(BASE, playbook),
        '-i', inventory,
        '-v',
        '--ssh-common-args=-o StrictHostKeyChecking=no ' +
        '-o UserKnownHostsFile=/dev/null'
    ]
    ansible_args.extend(args)
    ansible = local['ansible-playbook']
    exit_code = command_handler(ansible, ansible_args)
    return exit_code


def command_handler(command, args):
    """
    Generic function to run external programs.
    :param command: Exectuable to run
    :param args: arguments to be given to the external executable
    :return: The exit code of the external command, or exit code 255 if we are
    unable to determine the exit code
    """
    try:
        command.run(args, stdout=sys.stdout, stderr=sys.stderr)
        exit_code = 0
    except ProcessExecutionError as ex:
        print('Error encountered while executing command.',
              file=sys.stderr)
        exit_code = ex.retcode
    except Exception as ex:
        print('Unknown error occurred: {0}'.format(ex), file=sys.stderr)
        print_exc()
        exit_code = 255
    return exit_code



from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))
with open(path.join(here, 'README.md')) as f:
    description = f.read()

setup(
    name='cinch',
    version='1.4.0',
    description='Cinch continuous integration setup',
    long_description=description,
    url='https://github.com/RedHatQE/cinch',
    author='RedHatQE',
    license='GPLv3',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Topic :: Software Development :: Quality Assurance',
        'Topic :: Software Development :: Testing',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3'
    ],
    keywords='continuous integration, ci, jenkins',
    packages=find_packages(exclude=('library', 'bin')),
    include_package_data=True,
    install_requires=[
        'ansible>=2.4',
        'plumbum>=1.6.0'
    ],
    entry_points={
        'console_scripts': [
            'cinch=cinch.bin.entry_point:cinch',
            'teardown=cinch.bin.entry_point:teardown'
        ]
    },
    extras_require={
        'docs': [
            'sphinx_rtd_theme',
            'sphinx'
        ]
    }
)



from ansible.module_utils.basic import AnsibleModule
from tempfile import NamedTemporaryFile
import subprocess
import os


# Due to the nature of Ansible modules, this string cannot reasonably be
# handled outside of this file. Ansible modules must be self-contained single
# file scripts that can be uploaded as a unit. In this case, we need to drop
# this Groovy script into a temporary file to execute with the Jenkins CLI
# command
MYFILE = """
import hudson.model.User;
User u = User.get("{0}");
def p = u.getProperty(jenkins.security.ApiTokenProperty.class);
println p.getApiToken();
"""


def main():
    """
    Write out a groovy file that can be executed by the jenkins-cli.jar command
    which will print out the value of a specified user's API key. Currently the
    name of the user is not sanitized in any manner, so a value that includes
    Groovy escape characters or the double quote character (e.g. if the name
    includes the backslash character (\\) or the quotation mark character ("),
    or something else strange like a tab, newline, null, etc) then it will
    cause a problem with the Groovy code. Such usernames are generally not
    permissible in systems like Jenkins. If they are used, then the provider
    should escape those characters when they are passed into this module."""
    module = AnsibleModule(
        argument_spec={
            'user': {'required': True, 'type': 'str'},
            'cli_jar': {'default':
                        '/var/cache/jenkins/war/WEB-INF/jenkins-cli.jar'},
            'jenkins_url': {'default': 'http://localhost:8080'},
            'remoting': {'required': True, 'type': 'bool'},
            'java_command': {'type': 'str', 'default': '/usr/bin/java'}
        }
    )
    # Permits accessing args as object instead of dict
    args = type('Args', (object,), module.params)
    # Create a temporary place to put the Groovy code. False on the automatic
    # delete option, otherwise the call to the file .close() method would cause
    # it to be deleted, whereas we want the file to persist until after the
    # command is executed
    groovy = NamedTemporaryFile(delete=False)
    groovy.write(MYFILE.format(args.user))
    groovy.close()
    process_named_args = [args.java_command,
                          '-jar',
                          args.cli_jar,
                          '-s',
                          args.jenkins_url]
    # Append -remoting argument to named argument list depending on the version
    # of Jenkins
    if args.remoting:
        process_named_args.append('-remoting')
    process_positional_args = ['groovy', groovy.name]
    process = process_named_args + process_positional_args
    # The groovy code simply prints out the value of the API key, so we want
    # to be able to capture that output
    err, output = None, None
    p = subprocess.Popen(process,
                         stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE)
    try:
        output, err = p.communicate()
        os.unlink(groovy.name)
        # It's possible the Popen process has an error code for a whole host of
        # reasons
        if p.returncode == 0:
            module.exit_json(api_key=output.strip(),
                             err=err,
                             changed=False,
                             success=True)
        else:
            msg = "Error occurred while executing jenkins-cli.jar"
    except subprocess.CalledProcessError:
        msg = "Error received while attempting to execute Java"
        # If err and output are some type of empty, but not the empty string,
        # then we reached this point without any output. If they are the empty
        # string, then we reached this point but the subprocess output nothing
        # on the specified pipe. Providing this data, or a status message such
        # as these defaults, provides a better way for users to diagnose the
        # problems encountered
        if not err and err != "":
            err = "No stderr detected"
        if not output and output != "":
            output = "No stdout detected"
    # There are lots of reasons to fall through to here. But if we have, then
    # something has most definitely gone wrong. We should report on that
    module.fail_json(msg=msg,
                     stderr=err,
                     stdout=output,
                     api_key='')


main()
